import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;

import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * Map class
 * @author SergeiL
 */
public class PkNnMapper extends Mapper<LongWritable, Text, LongWritable, Text>
{
	private ArrayList<DataField> mTrainData;

	@Override
	protected void map(LongWritable key, Text value, Context context) throws IOException, InterruptedException
	{
		int[] dataArr = Helper.parseStringToInt(value.toString().split(";"));
		DataField data = new DataField(dataArr[0], dataArr[1], dataArr[2]);
		double distance = 0;
		ArrayList<DataField> fields = new ArrayList<DataField>();
		int neigh = context.getConfiguration().getInt("NUM_OF_NEIGHBOURS", 0);
		Hashtable<Long, Integer> groups = new Hashtable<Long, Integer>();
		DataField f;
		long maxGroup = Long.MIN_VALUE;
		int maxGroupCount = Integer.MIN_VALUE;

		for (DataField field : mTrainData)
		{
			distance = calcDistance(data.getX(), field.getX(), data.getY(), field.getY());
			field = new DataField(key.get(), distance, field.getGroup());
			fields.add(field);
		}
		
		Collections.sort(fields);

		for (int i = 0; i < neigh; i++)
		{
			f = fields.get(i);
			if(groups.containsKey(f.getGroup())) groups.put(f.getGroup(), groups.get(f.getGroup()) + 1);
			else groups.put(f.getGroup(), 1);
			if(groups.get(f.getGroup()) > maxGroupCount) 
			{
				maxGroupCount = groups.get(f.getGroup());
				maxGroup = f.getGroup();
			}
		}
		
		context.write(new LongWritable(data.getId()), new Text(String.valueOf(maxGroup)));
	}

	@Override
	protected void setup(Context context) throws IOException, InterruptedException
	{
		try
		{
			// DistributedCache.getCacheFiles()
			// Path my_path=new Path(uris[0].getPath());
			// http://mail-archives.apache.org/mod_mbox/hadoop-common-user/201007.mbox/%3C651981.45757.qm@web46311.mail.sp1.yahoo.com%3E
			// Path[] cacheFiles =
			// DistributedCache.getLocalCacheFiles(context.getConfiguration());
			URI[] cacheFiles = DistributedCache.getCacheFiles(context.getConfiguration());
			String trainDataPath = context.getConfiguration().get("TRAIN_DATA_HDFS_PATH");
			mTrainData = new ArrayList<DataField>();

			if (cacheFiles != null && cacheFiles.length > 0)
			{
				for (URI cachePath : cacheFiles)
				{
					if (cachePath.getPath().equals(trainDataPath))
					{
						Helper.loadTrainDataFile(cachePath.getPath(), mTrainData, context.getConfiguration());						
						break;
					}
				}
			}
		}
		catch (IOException ioe)
		{
			System.err.println("IOException reading from distributed cache");
			System.err.println(ioe.toString());
		}

		super.setup(context);
	}

	/**
	 * Calculate distance
	 * @param x1 - first x coordinate
	 * @param x2 - second x coordinate
	 * @param y1 - first y coordinate
	 * @param y2 - second y coordinate
	 * @return string
	 */
	private double calcDistance(long x1, long x2, long y1, long y2) {
		// Double.parseDouble(new
		// DecimalFormat("#.##").format(unformattedNumber));
		return Math.sqrt(Math.pow((x2 - x1), 2) + Math.pow((y2 - y1), 2));
	}
}