import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;

/**
 * Helper class
 * @author SergeiL
 */
public class Helper
{
	/**
	 * Method to read numbers (factors) from files in a folder in HDFS
	 */
	public static ArrayList<String> readResult(String pathStr, Configuration config) throws IOException
	{
		ArrayList<String> result = new ArrayList<String>();

		FileSystem fs = FileSystem.get(config);
		Path path = new Path(pathStr);

		if (fs.getFileStatus(path).isDir())
		{
			FileStatus[] listFiles = fs.listStatus(path);

			for (int i = 0; i < listFiles.length; i++)
			{
				try
				{
					if (listFiles[i].isDir()) continue;
					// Get the object of DataInputStream
					FSDataInputStream in = fs.open(listFiles[i].getPath());
					BufferedReader br = new BufferedReader(new InputStreamReader(in));
					String strLine;
					// Read File Line By Line
					while ((strLine = br.readLine()) != null) result.add(strLine);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		}

		return result;
	}

	/**
	 * Load train data file
	 * @param cachePath - cache path
	 * @return ArrayList<DataField>
	 * @throws IOException
	 */
	public static ArrayList<DataField> loadTrainDataFile(String cachePath, Configuration conf) throws IOException
	{
		// note use of regular java.io methods here - this is a local file now
		FileSystem fs = FileSystem.get(conf);
		BufferedReader reader = new BufferedReader(new InputStreamReader(fs.open(new Path(cachePath))));
		ArrayList<DataField> result;
		String sex;
		double heightMean;
		double weightMean;
		double footSizeMean;
		double heightVariance;
		double weightVariance;
		double footSizeVariance;

		try
		{
			String line;
			DataField field;
			String[] lineArr;
			result = new ArrayList<DataField>();

			while ((line = reader.readLine()) != null)
			{
				lineArr = line.split(";");
				sex = lineArr[0];
				heightMean = Double.parseDouble(lineArr[1]);
				heightVariance = Double.parseDouble(lineArr[2]);
				weightMean = Double.parseDouble(lineArr[3]);
				weightVariance = Double.parseDouble(lineArr[4]);
				footSizeMean = Double.parseDouble(lineArr[5]);
				footSizeVariance = Double.parseDouble(lineArr[6]);
				field = new DataField(sex, heightMean, heightVariance, weightMean, weightVariance, footSizeMean, footSizeVariance);
				result.add(field);
			}
		}
		finally
		{
			reader.close();
		}

		return result;
	}

	/**
	 * Text to data
	 * @param data - text data
	 * @return
	 */
	public static DataField textToData(Text data)
	{
		DataField result;
		String[] split = data.toString().split(";");
		double height = Double.parseDouble(split[1]);
		double weight = Double.parseDouble(split[2]);
		double footSize = Double.parseDouble(split[3]);
		result = new DataField(split[0], height, weight, footSize);
		return result;
	}
}