import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;

/**
 * Main class
 * @author SergeiL
 */
public class Main
{
	private static Configuration mConfig;

	//TODO: Get them from main function args
	private static String TEST_DATA_PATH_IN;
	private static String TEST_DATA_PATH_OUT;
	private static String TRAIN_DATA_HDFS_PATH;

	/**
	 * Main function
	 * @param args - arguments
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception
	{
		double start = System.currentTimeMillis();
		DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
		Date date = new Date();
		TEST_DATA_PATH_OUT = "Sergei_Laada/HadoopPnBm/" + dateFormat.format(date);
		
		TRAIN_DATA_HDFS_PATH = "Sergei_Laada/HadoopPnBm/trainData.csv";
		TEST_DATA_PATH_IN = "Sergei_Laada/HadoopPnBm/testData1m.csv";
		
		if(args != null && args.length > 0 && args[0] != null) TEST_DATA_PATH_IN = args[0];
		if(args != null && args.length > 1 && args[1] != null) TRAIN_DATA_HDFS_PATH = args[1];
		
		// trainData.csv testData.csv output 1 5
		Main prog = new Main();

		mConfig = new Configuration();
		mConfig.set("TRAIN_DATA_HDFS_PATH", TRAIN_DATA_HDFS_PATH);
		prog.run(TEST_DATA_PATH_IN, TEST_DATA_PATH_OUT);
		System.out.println("-----RESULT TIME: " + ((System.currentTimeMillis() - start) / 1000));
	}

	/**
	 * Run function
	 * @param pathin - path in
	 * @param pathout - path out
	 * @return int
	 * @throws Exception
	 */
	public int run(String pathin, String pathout) throws Exception
	{
		Job job = new Job(mConfig, "PnBm job");
		cacheTrainFile(job);
		job.setJarByClass(Main.class);

		job.setMapperClass(PnBmMapper.class);
		job.setReducerClass(PnBmReducer.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);

		job.setMapOutputKeyClass(LongWritable.class);
		job.setMapOutputValueClass(Text.class);

		job.setOutputKeyClass(LongWritable.class);
		job.setOutputValueClass(Text.class);

		job.setNumReduceTasks(0);

		FileInputFormat.addInputPath(job, new Path(pathin));
		FileOutputFormat.setOutputPath(job, new Path(pathout));

		boolean success = job.waitForCompletion(true);
		return success ? 0 : -1;
	}

	/**
	 * Cache train data
	 * @param job - job instance
	 * @throws IOException
	 */
	private static void cacheTrainFile(Job job) throws IOException
	{
		//FileSystem fs = FileSystem.get(job.getConfiguration());
		Path hdfsPath = new Path(TRAIN_DATA_HDFS_PATH);
		// upload the file to hdfs. Overwrite any existing copy.
		// http://stackoverflow.com/questions/15434709/checksum-exception-when-reading-from-or-copying-to-hdfs-in-apache-hadoop
		//fs.copyFromLocalFile(false, true, new Path(TRAIN_DATA_PATH), hdfsPath);
		DistributedCache.addCacheFile(hdfsPath.toUri(), job.getConfiguration());
	}
}