import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * Map class
 * @author SergeiL
 */
public class PnBmMapper extends Mapper<LongWritable, Text, LongWritable, Text>
{
	//Train data
	private ArrayList<DataField> mTrainData;

	@Override
	protected void map(LongWritable key, Text value, Context context) throws IOException, InterruptedException
	{
		DataField field = Helper.textToData(value);
		double pHeight;
		double pWeight;
		double pFootSize;
		double pMale = 0;
		double pFemale = 0;

		for (DataField element : mTrainData)
		{
			pHeight = (1 / Math.sqrt(2 * Math.PI * element.getVarianceHeight())) * Math.exp(-Math.pow(field.getHeight() - element.getMeanHeight(), 2) / (2 * element.getVarianceHeight()));
			pWeight = (1 / Math.sqrt(2 * Math.PI * element.getVarianceWeight())) * Math.exp(-Math.pow(field.getWeight() - element.getMeanWeight(), 2) / (2 * element.getVarianceWeight()));
			pFootSize = (1 / Math.sqrt(2 * Math.PI * element.getVarianceFootSize())) * Math.exp(-Math.pow(field.getFootSize() - element.getMeanFootSize(), 2) / (2 * element.getVarianceFootSize()));
			if (pFemale == 0) pFemale = 0.5 * pHeight * pWeight * pFootSize;
			else pMale = 0.5 * pHeight * pWeight * pFootSize;
		}

		if (pMale > pFemale) context.write(key, new Text("Male"));
		else context.write(key, new Text("Female"));
	}

	@Override
	protected void setup(Context context) throws IOException, InterruptedException
	{
		try
		{
			// DistributedCache.getCacheFiles()
			// Path my_path=new Path(uris[0].getPath());
			// http://mail-archives.apache.org/mod_mbox/hadoop-common-user/201007.mbox/%3C651981.45757.qm@web46311.mail.sp1.yahoo.com%3E
			// Path[] cacheFiles =
			// DistributedCache.getLocalCacheFiles(context.getConfiguration());
			URI[] cacheFiles = DistributedCache.getCacheFiles(context.getConfiguration());
			String trainDataPath = context.getConfiguration().get("TRAIN_DATA_HDFS_PATH");

			if (cacheFiles != null && cacheFiles.length > 0)
			{
				for (URI cachePath : cacheFiles)
				{
					if (cachePath.getPath().equals(trainDataPath))
					{
						mTrainData = Helper.loadTrainDataFile(cachePath.getPath(), context.getConfiguration());
						break;
					}
				}
			}
		}
		catch (IOException ioe)
		{
			System.err.println("IOException reading from distributed cache");
			System.err.println(ioe.toString());
		}

		super.setup(context);
	}
}