import java.io.IOException;
import java.util.ArrayList;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Reducer;

/**
 * Train data reduce class
 * @author SergeiL
 */
public class PnBmTrainReducer extends Reducer<Text, Text, Text, Text>
{
	@Override
	protected void reduce(Text key, Iterable<Text> values, Context context) throws IOException, InterruptedException
	{
		ArrayList<DataField> data = Helper.textToData(key.toString(), values);
		double[] heights = new double[data.size()];
		double[] weights = new double[data.size()];
		double[] footSizes = new double[data.size()];
		double heightMean = 0;
		double weightMean = 0;
		double footSizeMean = 0;
		double heightVariance = 0;
		double weightVariance = 0;
		double footSizeVariance = 0;

		for (int i = 0; i < data.size(); i++) heights[i] = data.get(i).getHeight();
		for (int i = 0; i < data.size(); i++) weights[i] = data.get(i).getWeight();
		for (int i = 0; i < data.size(); i++) footSizes[i] = data.get(i).getFootSize();

		heightMean = Helper.calcMean(heights);
		weightMean = Helper.calcMean(weights);
		footSizeMean = Helper.calcMean(footSizes);
		heightVariance = Helper.calcVariance(heights, heightMean);
		weightVariance = Helper.calcVariance(weights, weightMean);
		footSizeVariance = Helper.calcVariance(footSizes, footSizeMean);
		context.write(key, new Text(String.valueOf(heightMean) + Main.LINE_SEPARATOR + String.valueOf(heightVariance) + Main.LINE_SEPARATOR + String.valueOf(weightMean) + Main.LINE_SEPARATOR + String.valueOf(weightVariance) + Main.LINE_SEPARATOR + String.valueOf(footSizeMean) + Main.LINE_SEPARATOR + String.valueOf(footSizeVariance)));
	}
}