import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Class that holds information about each point
 * @author SergeiL
 */
public class Point implements Serializable, Comparable<Point> {

	private static final long serialVersionUID = 7767966786995509306L;
	// X coordinate of the point
	private int mCoordX;
	// Y coordinate of the point
	private int mCoordY;
	// Cluster cost when point is a medoid
	private double mClusterCost;
	private List<Point> mAssocPoints;

	/**
	 * Constructor
	 * @param x - x coordinate
	 * @param y - y coordinate
	 */
	public Point(int x, int y) {
		mCoordX = x;
		mCoordY = y;
		mClusterCost = 0;
		mAssocPoints = new ArrayList<Point>();
	}

	/**
	 * Get x coordinate
	 * @return int
	 */
	public int getX() {
		return mCoordX;
	}

	/**
	 * Get y coordinate
	 * @return int
	 */
	public int getY() {
		return mCoordY;
	}

	/**
	 * Convert class object to string
	 */
	public String toString() {
		String result = "x: " + mCoordX + " y: " + mCoordY;
		if(mAssocPoints.size() > 0) result += " assoc: ";
		
		for (int i = 0; i < mAssocPoints.size(); i++) 
		{
			result += " x: " + mAssocPoints.get(i).getX() + " y: " + mAssocPoints.get(i).getY();
		}
		
		if(mAssocPoints.size() > 0) result += " mClusterCost: " + mClusterCost;
		return result;
	}

	/**
	 * Get cluster cost
	 * @return double
	 */
	public double getClusterCost() {
		return mClusterCost;
	}

	/**
	 * Set cluster cost
	 * @param value - double
	 */
	public void setClusterCost(double value) {
		mClusterCost = value;
	}
	
	public void incrClusterCost(double value) {
		mClusterCost += value;
	}
	
	public void clearAssocPoints()
	{
		mAssocPoints.clear();
		mClusterCost = 0;
	}
	
	public void addAssocPoint(Point value)
	{
		mAssocPoints.add(value);
	}
	
	public List<Point> getAssocPoints()
	{
		return mAssocPoints;
	}

	@Override
	public int compareTo(Point p) {
		return mClusterCost > p.getClusterCost() ? 1 : (mClusterCost < p.getClusterCost() ? -1 : 0);
	}
}