import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Random;


public class Main {

	/**
	 * @param args
	 * @throws UnsupportedEncodingException 
	 * @throws FileNotFoundException 
	 */
	public static void main(String[] args) throws FileNotFoundException, UnsupportedEncodingException {
		generatePkNnData();
		generatePnBmData();
		generateClaraData();
		generatePkNnTrainData();
		//generatePkNnDefinedData();
		//generateClaraDefinedData();
		System.out.println("All done!");
	}
	
	private static void generatePkNnData() throws FileNotFoundException, UnsupportedEncodingException
	{
		PrintWriter writer = new PrintWriter("PkNnTestData.csv", "UTF-8");
		int idx = 0;
		int max = 1000;
		int iterations = 10000000;
		Random rng = new Random();
		int nextX;
		int nextY;
		
		for (int i = 0; i < iterations; i++) {
			nextX = rng.nextInt(max);
			nextY = rng.nextInt(max);
			writer.println(idx + ";" + nextX + ";" + nextY);
			idx++;
		}
		
		writer.close();
	}
	
	private static void generatePkNnDefinedData() throws FileNotFoundException, UnsupportedEncodingException
	{
		PrintWriter writer = new PrintWriter("PkNnDefinedTestData.csv", "UTF-8");
		
		generateDefinedPkNNTestData(writer, 1000, 100, 100, 300, 300, 0);
		generateDefinedPkNNTestData(writer, 1000, 700, 700, 900, 900, 1000);
		generateDefinedPkNNTestData(writer, 1000, 500, 500, 600, 600, 2000);
		generateDefinedPkNNTestData(writer, 100, 0, 0, 50, 50, 3000);
		generateDefinedPkNNTestData(writer, 1000, 800, 50, 1000, 150, 3100);
		generateDefinedPkNNTestData(writer, 100, 0, 0, 1000, 1000, 4100);
		
		writer.close();
	}
	
	private static void generateDefinedPkNNTestData(PrintWriter writer, int iter, int minX, int minY, int maxX, int maxY, int startIdx) throws FileNotFoundException, UnsupportedEncodingException
	{
		Random rng = new Random();
		int nextX;
		int nextY;
		
		for (int i = 0; i < iter; i++) {
			nextX = minX + rng.nextInt((maxX - minX) + 1);
			nextY = minY + rng.nextInt((maxY - minY) + 1);
			writer.println(startIdx + ";" + nextX + ";" + nextY);
			startIdx++;
		}
	}
	
	private static void generatePkNnTrainData() throws FileNotFoundException, UnsupportedEncodingException
	{
		PrintWriter writer = new PrintWriter("PkNnTrainData.csv", "UTF-8");
		int idx = 0;
		int max = 1000;
		int iterations = 1000;
		Random rng = new Random();
		int nextX;
		int nextY;
		int group;
		ArrayList<Integer> groups = new ArrayList<Integer>();
		
		for (int i = 0; i < iterations; i++) {
			nextX = rng.nextInt(max);
			nextY = rng.nextInt(max);
			group = getPkNNPointGroup(nextX, nextY);
			if(group == -1) continue;
			if(groups.indexOf(groups) != -1) groups.add(group);
			writer.println(idx + ";" + nextX + ";" + nextY + ";" + group);
			idx++;
			if(groups.size() < 5 && i == (iterations - 1)) iterations *= 2;
		}
		
		writer.close();
	}
	
	private static int getPkNNPointGroup(int x, int y)
	{
		if(x > 100 && y > 100 && x < 300 && y < 300) return 1;
		if(x > 700 && y > 700 && x < 900 && y < 900) return 2;	
		if(x > 500 && y > 500 && x < 600 && y < 600) return 3;	
		if(x > 0 && y > 0 && x < 50 && y < 50) return 4;	
		if(x > 800 && y > 50 && x < 1000 && y < 150) return 5;	
		return -1;
	}
	
	private static void generatePnBmData() throws FileNotFoundException, UnsupportedEncodingException
	{
		PrintWriter writer = new PrintWriter("PnBmTestData.csv", "UTF-8");
		DecimalFormat df = new DecimalFormat("0.00");
		int idx = 0;
		int minHeight = 4;
		int maxHeight = 6 - minHeight;
		int minWeight = 100;
		int maxWeight = 250 - minWeight;
		int minFootSize = 3;
		int maxFootSize = 13 - minFootSize;
		int iterations = 100000000;
		String nextHeight;
		String nextWeight;
		String nextFootSize;
		
		for (int i = 0; i < iterations; i++) {
			nextHeight = df.format(minHeight + Math.random() * maxHeight).replace(',', '.');
			nextWeight = df.format(minWeight + Math.random() * maxWeight).replace(',', '.');
			nextFootSize = df.format(minFootSize + Math.random() * maxFootSize).replace(',', '.');
			writer.println(idx + ";" + nextHeight + ";" + nextWeight + ";" + nextFootSize);
			idx++;
		}
		
		writer.close();
	}
	
	private static void generateClaraDefinedData() throws FileNotFoundException, UnsupportedEncodingException
	{
		PrintWriter writer = new PrintWriter("ClaraDefinedTestData.csv", "UTF-8");

		generateDefinedClaraTestData(writer, 1000, 200, 200, 400, 400);
		generateDefinedClaraTestData(writer, 1000, 700, 700, 900, 900);
		generateDefinedClaraTestData(writer, 1000, 500, 500, 600, 600);
		generateDefinedClaraTestData(writer, 1000, 0, 0, 100, 100);
		generateDefinedClaraTestData(writer, 1000, 800, 50, 1000, 150);
		generateDefinedClaraTestData(writer, 100, 0, 0, 900, 900);
		
		writer.close();
	}
	
	private static void generateClaraData() throws FileNotFoundException, UnsupportedEncodingException
	{
		int max = 1000;
		int iterations = 1000000;
		int workers = 1;
		int datas = iterations / workers;		
		int dataAmount = 1;
		Random rng = new Random();
		int nextX;
		int nextY;
		File dir = new File("testData" + dataAmount + "m.csv");
		dir.mkdir();
		PrintWriter writer = new PrintWriter("testData" + dataAmount + "m.csv/ClaraTestData0.csv", "UTF-8");
		
		for (int i = 0; i < iterations; i++) {			
			nextX = rng.nextInt(max);
			nextY = rng.nextInt(max);
			writer.println(nextX + ";" + nextY);
			if(i % datas == 0) {
				writer.close();
				writer = new PrintWriter("testData" + dataAmount + "m.csv/ClaraTestData" + i + ".csv", "UTF-8");
			}
		}
		
		writer.close();
	}
	
	private static void generateDefinedClaraTestData(PrintWriter writer, int iter, int minX, int minY, int maxX, int maxY) throws FileNotFoundException, UnsupportedEncodingException
	{
		Random rng = new Random();
		int nextX;
		int nextY;
		
		for (int i = 0; i < iter; i++) {
			nextX = minX + rng.nextInt((maxX - minX) + 1);
			nextY = minY + rng.nextInt((maxY - minY) + 1);
			writer.println(nextX + ";" + nextY);
		}
	}
}
